from typing import Text

def convert_zero_consonant(pinyin: Text) -> Text: ...


def convert_uv(pinyin: Text) -> Text: ...


def convert_iou(pinyin: Text) -> Text: ...


def convert_uei(pinyin: Text) -> Text: ...


def convert_uen(pinyin: Text) -> Text: ...


def convert_finals(pinyin: Text) -> Text: ...
