from typing import Text

def get_initials(pinyin: Text, strict: bool) -> Text: ...


def get_finals(pinyin: Text, strict: bool) -> Text: ...


def replace_symbol_to_number(pinyin: Text) -> Text: ...


def replace_symbol_to_no_symbol(pinyin: Text) -> Text: ...


def has_finals(pinyin: Text) -> bool: ...
